CREATE TYPE Address AS OBJECT (
         STREET_NO NUMBER,
         STREET_NAME VARCHAR2(25),
	 CITY VARCHAR2(15),
	 ZIP  VARCHAR2(10),
         STATE VARCHAR2(2)
     );

CREATE TABLE DEPARTMENT (
	DNAME VARCHAR2(30) PRIMARY KEY,
	WEB_ADDRESS VARCHAR2(30),
	MAIN_PHONE NUMBER,
	MAIN_ADDRESS Address,
	EMAIL VARCHAR2(25));

CREATE TABLE BUILDING (
	BNAME VARCHAR2(30) PRIMARY KEY,
	code VARCHAR2(3),
	N_ROOMS NUMBER,
	N_FLOORS NUMBER,
	MAILING_ADDRESS Address);

CREATE TABLE ROOM (
	BNAME VARCHAR2(30) REFERENCES BUILDING(BNAME) ON DELETE CASCADE,
	RNUMBER NUMBER,
	RTYPE VARCHAR2(1),
	AREA NUMBER,
	N_PHONE NUMBER,
	N_INTJK NUMBER,
	PRIMARY KEY(BNAME,RNUMBER)
	);
	
CREATE TABLE ROOM_READINGS (
	BNAME VARCHAR2(30) REFERENCES BUILDING(BNAME) ON DELETE CASCADE,
	RNUMBER NUMBER,
	READINGS  MEASUREMENT,
	PRIMARY KEY(BNAME,RNUMBER,READINGS.MTIME),
	FOREIGN KEY(BNAME,RNUMBER) REFERENCES ROOM(BNAME,RNUMBER)
	);

CREATE TABLE ROOMPHONE (
	BNAME VARCHAR2(30) REFERENCES BUILDING(BNAME) ON DELETE CASCADE,
	RNUMBER NUMBER NOT NULL,
	PHONE NUMBER,
	PRIMARY KEY(RNUMBER,PHONE));

CREATE OR REPLACE TYPE EQUIPMENT_TY AS OBJECT (
	EQPID NUMBER,
	SERVICETAG VARCHAR2(30),
	PURCHASEDATE DATE,
	EQPTYPE VARCHAR2(1),
	BNAME VARCHAR2(30),
	RNUMBER NUMBER) NOT FINAL;

CREATE TYPE PRINTER_TY UNDER EQUIPMENT_TY (
	IP VARCHAR2(30),
	TYPE VARCHAR2(5));

CREATE TYPE COMPUTER_TY UNDER EQUIPMENT_TY (
	BRANDNAME VARCHAR2(20),
	CPUMODEL VARCHAR2(15),
	IPADDRESS VARCHAR2(30),
	MACADDRESS VARCHAR2(30)) NOT FINAL;

CREATE TYPE SERVER_TY UNDER COMPUTER_TY (
	SERVICE VARCHAR2(15));

CREATE TABLE EQUIPMENT OF EQUIPMENT_TY(
	EQPID PRIMARY KEY,
	FOREIGN KEY(BNAME,RNUMBER) REFERENCES ROOM(BNAME,RNUMBER) ON DELETE CASCADE);

CREATE TABLE PRINTER OF PRINTER_TY(
	EQPID PRIMARY KEY);

CREATE TABLE COMPUTER OF COMPUTER_TY(
	EQPID PRIMARY KEY);

CREATE TABLE SERVER OF SERVER_TY(
	EQPID PRIMARY KEY);
	
CREATE TYPE PERSON_TY AS OBJECT (
	SSN VARCHAR2(11),
	EMPNAME VARCHAR2(50),
	EMPADDRESS Address,
	YOB NUMBER,
	AGE NUMBER,
	MEMBER FUNCTION GET_AGE RETURN NUMBER);

CREATE TABLE DEPT_EQUIP (
	DNAME VARCHAR2(30) REFERENCES DEPARTMENT(DNAME) ON DELETE CASCADE,
	EQPID NUMBER REFERENCES EQUIPMENT(EQPID) ON DELETE CASCADE,
	PRIMARY KEY(DNAME,EQPID));

CREATE TABLE EMP_DEPT_ROOM (
	SSN VARCHAR2(11) REFERENCES EMPLOYEE(SSN) ON DELETE CASCADE,
	DNAME VARCHAR2(30) REFERENCES DEPARTMENT(DNAME) ON DELETE CASCADE,
	BNAME VARCHAR2(30) REFERENCES BUILDING(BNAME) ON DELETE CASCADE,
	RNUMBER NUMBER NOT NULL,
	WORKSCHED SCHEDULE,
	PRIMARY KEY(SSN,DNAME,BNAME,RNUMBER,WORKSCHED.DAY,WORKSCHED.STARTTIME,WORKSCHED.ENDTIME));
	

CREATE TABLE EMPLOYEE OF PERSON_TY(
	SSN PRIMARY KEY);

CREATE TYPE MEASUREMENT AS OBJECT (
	MTIME TIMESTAMP,
	SOUND NUMBER,
	TEMPERATURE NUMBER,
	LIGHT NUMBER)FINAL;

CREATE TYPE SCHEDULE AS OBJECT (
	DAY VARCHAR2(10),
	STARTTIME VARCHAR2(8),
	ENDTIME VARCHAR2(8)
	)FINAL;

CREATE OR REPLACE TYPE BODY PERSON_TY AS
MEMBER FUNCTION GET_AGE RETURN NUMBER IS
BEGIN
RETURN TO_CHAR(SYSDATE,'YYYY')-YOB;
END GET_AGE;
END;

INSERT INTO DEPARTMENT VALUES ('COMPUTER SCIENCE','www.cs.usc.edu',2137404496,address(7560,'BLOOM WALK','LOS ANGELES','90007','CA'),'cs@usc.edu');
INSERT INTO DEPARTMENT VALUES ('ELECTRICAL ENGINEERING','www.ee.usc.edu',2137404345,address(2598,'BLOOM WALK','LOS ANGELES','90007','CA'),'ee@usc.edu');
INSERT INTO DEPARTMENT VALUES ('AEROSPACE','www.as.usc.edu',2137406580,address(8763,'WATT WAY','LOS ANGELES','90007','CA'),'as@usc.edu');
INSERT INTO DEPARTMENT VALUES ('BIOTECHNOLOGY','www.bt.usc.edu',2137405960,address(2560,'WATT WAY','LOS ANGELES','90007','CA'),'bt@usc.edu');
INSERT INTO DEPARTMENT VALUES ('CHEMICAL','www.chem.usc.edu',2137404520,address(2560,'CHILDS WAY','LOS ANGELES','90007','CA'),'chem@usc.edu');

INSERT INTO DEPT_EQUIP VALUES ('COMPUTER SCIENCE',1000);
INSERT INTO DEPT_EQUIP VALUES ('COMPUTER SCIENCE',1001);
INSERT INTO DEPT_EQUIP VALUES ('COMPUTER SCIENCE',1002);
INSERT INTO DEPT_EQUIP VALUES ('COMPUTER SCIENCE',1003);
INSERT INTO DEPT_EQUIP VALUES ('ELECTRICAL ENGINEERING',1003);
INSERT INTO DEPT_EQUIP VALUES ('ELECTRICAL ENGINEERING',1002);
INSERT INTO DEPT_EQUIP VALUES ('ELECTRICAL ENGINEERING',1001);
INSERT INTO DEPT_EQUIP VALUES ('AEROSPACE',1001);
INSERT INTO BUILDING VALUES ('SALVATORY','SAL',50,3,address(7560,'BLOOM WALK','LOS ANGELES','90007','CA'));
INSERT INTO BUILDING VALUES ('OLIN HALL','OHE',60,4,address(2560,'CHILDS WAY','LOS ANGELES','90007','CA'));
INSERT INTO BUILDING VALUES ('EE BUILDING','EEB',100,5,address(2598,'BLOOM WALK','LOS ANGELES','90007','CA'));
INSERT INTO BUILDING VALUES ('HEDCO NEUROSCIENCES','HNS',45,3,address(2560,'WATT WAY','LOS ANGELES','90007','CA'));
INSERT INTO BUILDING VALUES ('PHILIPS HALL','PHE',34,2,address(8763,'WATT WAY','LOS ANGELES','90007','CA'));

INSERT INTO EMPLOYEE VALUES ('6125697849','Todd Fuerer',address(2598,'BLOOM WALK','LOS ANGELES','90007','CA'),1952,0);
INSERT INTO EMPLOYEE VALUES ('612569821','Martin X Truman',address(4156,'BLOOM WALK','LOS ANGELES','90007','CA'),1974,0);
INSERT INTO EMPLOYEE VALUES ('616565698','Felix N King',address(4156,'Jefferson','LOS ANGELES','90007','CA'),1948,0);
INSERT INTO EMPLOYEE VALUES ('610568796','Rahul Banerjee',address(1013,'Ellendale','LOS ANGELES','90007','CA'),1980,0);
INSERT INTO EMPLOYEE VALUES ('620568216','William Johnson',address(2036,'34th','LOS ANGELES','90007','CA'),1978,0);
INSERT INTO EMPLOYEE VALUES ('620567789','Susan O Henry',address(2121,'39th','LOS ANGELES','90007','CA'),1969,0);
INSERT INTO EMPLOYEE E VALUES ('618567789','Yuan Li',address(3001,'Adams','LOS ANGELES','90007','CA'),1982,0);
INSERT INTO EMPLOYEE E VALUES ('618565569','Rohan Mehta',address(2569,'Beranger','San Jose','90007','CA'),1970,0);
INSERT INTO EMPLOYEE E VALUES ('640569234','Roger Simpson',address(2569,'Beranger','San Francisco','90007','CA'),1980,0);

INSERT INTO EMP_DEPT_ROOM VALUES ('6125697849','COMPUTER SCIENCE','SALVATORY',101,SCHEDULE('MONDAY','10:30:00','12:00:00'));
INSERT INTO EMP_DEPT_ROOM VALUES ('6125697849','ELECTRICAL ENGINEERING','EE BUILDING',101,SCHEDULE('TUESDAY','08:30:00','12:00:00'));
INSERT INTO EMP_DEPT_ROOM VALUES ('610568796','ELECTRICAL ENGINEERING','EE BUILDING',415,SCHEDULE('TUESDAY','08:30:00','12:00:00'));
INSERT INTO EMP_DEPT_ROOM VALUES ('610568796','AEROSPACE','PHILIPS HALL',415,SCHEDULE('WEDNESDAY','08:30:00','12:00:00'));

INSERT INTO ROOM VALUES ('SALVATORY',100,'C',1200,4,6);
INSERT INTO ROOM VALUES ('EE BUILDING',105,'C',950,4,6);
INSERT INTO ROOM VALUES ('EE BUILDING',101,'C',950,4,6);
INSERT INTO ROOM VALUES ('EE BUILDING',415,'C',950,4,6);
INSERT INTO ROOM VALUES ('PHILIPS HALL,415,'O',800,4,8);

INSERT INTO ROOM_READINGS VALUES ('SALVATORY',100,MEASUREMENT(TO_TIMESTAMP( 2004-02-28 10:30:00 , YYYY-MM-DD HH:MI:SS ),500,52,450));
INSERT INTO ROOM_READINGS VALUES ('SALVATORY',100,MEASUREMENT(TO_TIMESTAMP( 2004-02-28 11:30:00 , YYYY-MM-DD HH:MI:SS ),580,56,600));
INSERT INTO ROOM_READINGS VALUES ('SALVATORY',100,MEASUREMENT(TO_TIMESTAMP( 2004-02-29 12:30:00 , YYYY-MM-DD HH:MI:SS ), 0,58,0));
INSERT INTO ROOM_READINGS VALUES ('SALVATORY',100,MEASUREMENT(TO_TIMESTAMP( 2004-02-29 07:30:00 , YYYY-MM-DD HH:MI:SS ), 0,40,380));
INSERT INTO ROOM_READINGS VALUES 
('EE BUILDING',105,MEASUREMENT(TO_TIMESTAMP( 2004-02-29 07:30:00 , YYYY-MM-DD HH:MI:SS ),0,40,600));
INSERT INTO ROOM_READINGS VALUES 
('EE BUILDING',101,MEASUREMENT(TO_TIMESTAMP( 2004-02-29 07:30:00 , YYYY-MM-DD HH:MI:SS ),250,40,710));
INSERT INTO ROOM_READINGS VALUES 
('EE BUILDING',101,MEASUREMENT(TO_TIMESTAMP( 2004-02-29 11:30:00 , YYYY-MM-DD HH:MI:SS ),450,58,710));
INSERT INTO ROOM_READINGS VALUES 
('EE BUILDING',415,MEASUREMENT(TO_TIMESTAMP( 2004-03-06 12:30:00 , YYYY-MM-DD HH:MI:SS ),450,65,760));
INSERT INTO ROOM_READINGS VALUES 
('EE BUILDING',415,MEASUREMENT(TO_TIMESTAMP( 2004-03-06 07:56:00 , YYYY-MM-DD HH:MI:SS ),0,48,0));
INSERT INTO ROOM_READINGS VALUES 
('PHILIPS HALL',415,MEASUREMENT(TO_TIMESTAMP( 2004-03-06 10:56:00 , YYYY-MM-DD HH:MI:SS ),150,58,0));
INSERT INTO ROOM_READINGS VALUES 
('PHILIPS HALL',415,MEASUREMENT(TO_TIMESTAMP( 2004-03-06 12:56:00 , YYYY-MM-DD HH:MI:SS ),150,69,200));
INSERT INTO EQUIPMENT VALUES (1000,SERVICED ON 12/12/2004 BY IBM,10-FEB-2003,C,SALVATORY,100);
INSERT INTO EQUIPMENT VALUES (1001,SERVICED ON 02/12/2004 BY IBM,10-DEC-2003,C,SALVATORY,101);
INSERT INTO EQUIPMENT VALUES (1002,SERVICED ON 02/20/2004 BY MS,10-DEC-2003,S,SALVATORY,101);
INSERT INTO EQUIPMENT VALUES (1003,SERVICED ON 02/20/2004 BY MS,10-DEC-2003,S,SALVATORY,100);

INSERT INTO COMPUTER VALUES (1000,SERVICED ON 12/12/2004 BY IBM,10-FEB-2003,C, SALVATORY,100,IBM,PENTIUM III,127.128.45.98,127.0.0.1);
INSERT INTO COMPUTER VALUES (1001,SERVICED ON 02/12/2004 BY IBM,10-DEC-2003,C, SALVATORY,101,IBM,PENTIUM III,127.128.45.125,127.0.0.2);
INSERT INTO SERVER VALUES (1002,SERVICED ON 02/20/2004 BY MS,10-DEC-2003,S, SALVATORY,101,IBM,PENTIUM III,127.128.45.120,127.0.0.10,DNS);
INSERT INTO SERVER VALUES (1003,SERVICED ON 02/20/2004 BY MS,10-DEC-2003,S, SALVATORY,100,IBM,PENTIUM G40,127.128.45.65,127.0.0.20,FILE);

INSERT INTO ROOM VALUES ('SALVATORY',101,L,800,2,4);
INSERT INTO ROOM_READINGS VALUES ('SALVATORY',101,MEASUREMENT(TO_TIMESTAMP( 2004-02-28 08:30:00 , YYYY-MM-DD HH:MI:SS ),400,52,350));
INSERT INTO ROOM_READINGS VALUES ('SALVATORY',101,MEASUREMENT(TO_TIMESTAMP( 2004-02-28 11:30:00 , YYYY-MM-DD HH:MI:SS ),650,54,550));